==> pywrangle-0.2.47.tar.gz: pywrangle-0.2.47/pywrangle/missing_data.py <==
"""/**
 * @author [Jai Miles]
 * @email [jaimiles23@gmail.com]
 * @create date 2020-08-20 16:08:16
 * @modify date 2020-08-20 23:16:16
 * @desc [
    Contains auxiliary functions for cleaning missing data.
 ]
 */
"""


##########
# Imports
##########

from typing import (
    List,
    Tuple,
)


import numpy as np
import seaborn as sns
import pandas as pd


try:
    import printing
    import aux_functions
except (ModuleNotFoundError):
    from pywrangle import printing
    from pywrangle import aux_functions


##########
# Print Nulls per column
##########

def show_col_nulls(
    df,
    show_null_heatmap: bool = True,
) -> None:
    """
    Calculates number of null values in each column and prints result.
    
    Calls auxiliary functions:
    - _count_column_nulls

    ## Tests
    >>> df_winereviews = pd.read_csv("../input/wine-reviews/winemag-data_first150k.csv")
    >>> show_col_nulls(df_winereviews)
        89977	region_2
        45735	designation
        25060	region_1
        13695	price
        5	    country
        5	    province
        0	    description
        0	    points
        0	    variety
        0	    winery
    https://www.kaggle.com/jaimiles23/wine-tasting

    TODO:
        - Update testing documentation to be compliant with pytests
        - May like to add corr matrix of NULL values - need to test!
            - https://stats.stackexchange.com/questions/464663/how-to-calculate-nullity-correlation-matrix
            - https://towardsdatascience.com/better-heatmaps-and-correlation-matrix-plots-in-python-41445d0f2bec
            
    """


    def _count_column_nulls(df) -> List[ Tuple[ int, str]]:
        """
        Returns list of tuples (str, int) each column and its number of nulls.
        """
        col_nulls: List[ Tuple[int, str]] = []

        ## Create tuples of number of nulls in respective column.
        for col in df.columns:
            num_null = df[col].isna().sum()
            col_nulls.append( (col, num_null))

        col_nulls.sort(key = lambda x: x[1], reverse = True)
        return col_nulls
    
    
    ## Null values
    column_nulls = _count_column_nulls(df)

    ## Columns length
    max_colname_length = aux_functions.get_max_colname_length(df, "Null")

    ## Print headers
    printing._print_headers_colname_oneattr(
        df= df, 
        attr_header= "Null",
        max_colname_length= max_colname_length,
    )
    printing._print_tuple_with_colname_spacing(
        column_nulls, max_colname_length= max_colname_length)

    ## Null heatmap
    if show_null_heatmap:
        sns.heatmap( df.isnull(), cbar = False)
    return

==> pywrangle-0.2.51.tar.gz: pywrangle-0.2.51/pywrangle/missing_data.py <==
"""/**
 * @author [Jai Miles]
 * @email [jaimiles23@gmail.com]
 * @create date 2020-08-20 16:08:16
 * @modify date 2020-08-20 23:16:16
 * @desc [
    Contains auxiliary functions for cleaning missing data.
 ]
 */
"""


##########
# Imports
##########

from typing import (
    List,
    Tuple,
)


import numpy as np
import seaborn as sns
import pandas as pd


try:
    import printing
    import aux_functions
except (ModuleNotFoundError):
    from pywrangle import printing
    from pywrangle import aux_functions


##########
# Print Nulls per column
##########

def show_col_nulls(
    df,
    show_null_heatmap: bool = False,
    show_null_corr_matrix: bool = False,
) -> None:
    """
    Calculates number of null values in each column and prints result.
    
    Calls auxiliary functions:
    - _count_column_nulls

    ## Tests
    >>> df_winereviews = pd.read_csv("../input/wine-reviews/winemag-data_first150k.csv")
    >>> show_col_nulls(df_winereviews)
        89977	region_2
        45735	designation
        25060	region_1
        13695	price
        5	    country
        5	    province
        0	    description
        0	    points
        0	    variety
        0	    winery
    https://www.kaggle.com/jaimiles23/wine-tasting

    TODO:
        - Update testing documentation to be compliant with pytests            
    """


    def _count_column_nulls(df) -> List[ Tuple[ int, str]]:
        """
        Returns list of tuples (str, int) each column and its number of nulls.
        """
        col_nulls: List[ Tuple[int, str]] = []

        ## Create tuples of number of nulls in respective column.
        for col in df.columns:
            num_null = df[col].isna().sum()
            col_nulls.append( (col, num_null))

        col_nulls.sort(key = lambda x: x[1], reverse = True)
        return col_nulls
    

    def get_null_corr_matrix(df, col_nulls: List[ Tuple[int, str]]) -> None:
        """Prints Null correlation matrix for variables with Null values.
        
        - https://stats.stackexchange.com/questions/464663/how-to-calculate-nullity-correlation-matrix
        
        ## May like to improve readability with this link:
        - https://towardsdatascience.com/better-heatmaps-and-correlation-matrix-plots-in-python-41445d0f2bec
        """
        ## Remove columns with no null values
        for col, null in col_nulls:
            if null == 0:
                df.drop(
                    col,
                    axis = 1,
                    inplace = True
                )
                print(f"{col} has {null} null values.")
        df.isnull().corr()
        

    ## Null values
    column_nulls = _count_column_nulls(df)

    ## Columns length
    max_colname_length = aux_functions.get_max_colname_length(df, "Null")

    ## Print headers
    printing._print_headers_colname_oneattr(
        df= df, 
        attr_header= "Null",
        max_colname_length= max_colname_length,
    )
    printing._print_tuple_with_colname_spacing(
        column_nulls, max_colname_length= max_colname_length)

    ## Null heatmap
    if show_null_heatmap:
        sns.heatmap( df.isnull(), cbar = False)
    
    ## Correlation matrix
    if show_col_nulls:
        get_null_corr_matrix(df, column_nulls)
    
    return
